from __future__ import unicode_literals

from django.db import models
from django.template.defaultfilters import truncatechars
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class Tag(models.Model):
    name = models.CharField(max_length=200, unique=True)

    class Meta:
        ordering = ('name',)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return 'https://www.instagram.com/explore/tags/%s/' % (self.name,)


@python_2_unicode_compatible
class Photo(models.Model):
    photo_id = models.CharField(max_length=100, unique=True)
    user = models.CharField(max_length=30, db_index=True)
    image = models.URLField()
    image_width = models.PositiveIntegerField()
    image_height = models.PositiveIntegerField()
    image_file = models.ImageField(upload_to='quickphotos/photo', blank=True)
    created = models.DateTimeField(db_index=True)
    caption = models.TextField()
    tags = models.ManyToManyField(Tag, blank=True)
    link = models.URLField()
    like_count = models.PositiveIntegerField(default=0)
    comment_count = models.PositiveIntegerField(default=0)

    class Meta:
        ordering = ('-created',)

    def __str__(self):
        if self.caption:
            return '%s - %s' % (self.user, truncatechars(self.caption, 50))
        else:
            return self.user

    def get_absolute_url(self):
        return self.link


@python_2_unicode_compatible
class Like(models.Model):
    user = models.CharField(max_length=30, db_index=True)
    photo = models.ForeignKey(Photo)

    class Meta:
        ordering = ('-photo',)
        unique_together = (
            ('user', 'photo'),
        )

    def __str__(self):
        return '%s' % (self.photo,)



from django.core.management.base import BaseCommand
from django.utils.six.moves import input
from instagram.client import InstagramAPI


def get_auth_tokens(stdout):
    stdout.write('Please enter the following Instagram client details\n\n')
    client_id = input('Client ID: ').strip()
    client_secret = input('Client Secret: ').strip()
    redirect_uri = input('Redirect URI: ').strip()
    raw_scope = input(
        'Requested scope (separated by spaces, blank for just basic read): ').strip()
    scope = raw_scope.split(' ')

    # For basic, API seems to need to be set explicitly
    if not scope or scope == ['']:
        scope = ['basic']

    api = InstagramAPI(client_id=client_id, client_secret=client_secret, redirect_uri=redirect_uri)
    redirect_uri = api.get_authorize_login_url(scope=scope)

    stdout.write('\nVisit this page and authorize access in your browser:\n\n%s\n\n' % redirect_uri)

    code = input('Paste in code in query string after redirect: ').strip()

    access_token = api.exchange_code_for_access_token(code)
    stdout.write('Access token:\n\n%s\n\n' % (access_token,))


class Command(BaseCommand):
    help = 'Generate access token needed for Instagram'

    def handle(self, **options):
        get_auth_tokens(self.stdout)



from django.contrib import admin

from .models import Photo


@admin.register(Photo)
class PhotoAdmin(admin.ModelAdmin):
    list_display = ('user', 'caption', 'created')
    list_filter = ('created',)
    date_hierarchy = 'created'
    readonly_fields = (
        'photo_id', 'user', 'image', 'created', 'caption', 'tags', 'link', 'like_count',
        'comment_count')
    fieldsets = (
        (None, {
            'fields': readonly_fields,
        }),
    )

    def has_add_permission(self, request):
        return False



import hashlib
from tempfile import TemporaryFile

from django.core.files import File
from django.utils.timezone import make_aware, utc
from PIL import Image
import requests

from .models import Like, Photo, Tag


def photo_tags(obj, tags):
    for tag in tags:
        name = tag.name.lower()

        # No sane limit on tags - so we enforce one here by avoiding them
        if len(name) > 200:
            continue

        tag_obj, created = Tag.objects.get_or_create(name=name)

        obj.tags.add(tag_obj)


def update_photos(photos, download=False):
    obj_list = []

    for i in photos:
        image = i.images['standard_resolution']

        if i.caption:
            caption = i.caption.text
        else:
            caption = ''

        obj, created = Photo.objects.update_or_create(photo_id=i.id, defaults={
            'user': i.user.username,
            'image': image.url,
            'image_width': image.width,
            'image_height': image.height,
            'created': make_aware(i.created_time, utc),
            'caption': caption,
            'link': i.link,
            'like_count': i.like_count,
            'comment_count': i.comment_count,
        })

        if download and not obj.image_file:
            with TemporaryFile() as temp_file:
                image_file = File(temp_file)

                # Download the file
                r = requests.get(image.url, stream=True)
                r.raise_for_status()

                for chunk in r.iter_content(4096):
                    image_file.write(chunk)

                # Get Pillow to look at it
                image_file.seek(0)
                pil_image = Image.open(image_file)
                image_name = '%s.%s' % (
                    hashlib.md5(image.url.encode()).hexdigest(), pil_image.format.lower())

                # Save the file
                image_file.seek(0)
                obj.image_file.save(image_name, image_file, save=True)

        # Add tags
        photo_tags(obj=obj, tags=i.tags)

        obj_list.append(obj)

    return obj_list


def update_likes(user, photos, download=False):
    obj_list = update_photos(photos=photos, download=download)

    for photo in obj_list:
        Like.objects.get_or_create(user=user, photo=photo)

    return obj_list



from django import template

from quickphotos.models import Photo, Tag

register = template.Library()


@register.assignment_tag
def get_latest_photos(*args, **kwargs):
    limit = kwargs.pop('limit', None)
    liked_by = kwargs.pop('liked_by', None)
    tag = kwargs.pop('tag', None)
    photos = Photo.objects.all()

    if liked_by:
        photos = photos.filter(like__user=liked_by)

    if tag:
        try:
            tag_obj = Tag.objects.get(name=tag)
            photos = photos.filter(tags=tag_obj)
        except Tag.DoesNotExist:
            photos = photos.none()

    if args:
        photos = photos.filter(user__in=args)

    if limit is not None:
        photos = photos[:limit]

    return photos



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('quickphotos', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='photo',
            old_name='thumb',
            new_name='image',
        ),
        migrations.RenameField(
            model_name='photo',
            old_name='thumb_height',
            new_name='image_height',
        ),
        migrations.RenameField(
            model_name='photo',
            old_name='thumb_width',
            new_name='image_width',
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('quickphotos', '0002_thumb_to_standard_image'),
    ]

    operations = [
        migrations.AddField(
            model_name='photo',
            name='comment_count',
            field=models.PositiveIntegerField(default=0),
        ),
        migrations.AddField(
            model_name='photo',
            name='like_count',
            field=models.PositiveIntegerField(default=0),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('quickphotos', '0005_tags'),
    ]

    operations = [
        migrations.CreateModel(
            name='Like',
            fields=[
                ('id', models.AutoField(serialize=False, auto_created=True, verbose_name='ID', primary_key=True)),
                ('user', models.CharField(max_length=30, db_index=True)),
                ('photo', models.ForeignKey(to='quickphotos.Photo')),
            ],
            options={
                'ordering': ('-photo',),
            },
        ),
        migrations.AlterUniqueTogether(
            name='like',
            unique_together=set([('user', 'photo')]),
        ),
    ]



from django.conf import settings
from django.core.management.base import BaseCommand
from django.db import transaction
from instagram.client import InstagramAPI

from quickphotos.utils import update_likes


@transaction.atomic
def update_user_likes(download):
    api = InstagramAPI(
        access_token=settings.INSTAGRAM_ACCESS_TOKEN,
        client_secret=settings.INSTAGRAM_CLIENT_SECRET)

    # Figure out the logged in user
    user = api.user()
    username = user.username

    # Find the media the user has liked
    liked_media, next = api.user_liked_media()
    update_likes(user=username, photos=liked_media, download=download)


class Command(BaseCommand):
    help = 'Download and store the latest liked photos'

    def add_arguments(self, parser):
        parser.add_argument(
            '--download-photos', action='store_true',
            help='Download images from photos and store locally')

    def handle(self, **options):
        update_user_likes(download=options['download_photos'])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Photo',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, primary_key=True, auto_created=True)),
                ('photo_id', models.CharField(unique=True, max_length=100)),
                ('user', models.CharField(db_index=True, max_length=30)),
                ('thumb', models.URLField()),
                ('thumb_width', models.PositiveIntegerField()),
                ('thumb_height', models.PositiveIntegerField()),
                ('created', models.DateTimeField(db_index=True)),
                ('caption', models.TextField()),
                ('link', models.URLField()),
            ],
            options={
                'ordering': ('-created',),
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('quickphotos', '0004_photo_image_file'),
    ]

    operations = [
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('id', models.AutoField(verbose_name='ID', auto_created=True, serialize=False, primary_key=True)),
                ('name', models.CharField(max_length=200, unique=True)),
            ],
            options={
                'ordering': ('name',),
            },
        ),
        migrations.AddField(
            model_name='photo',
            name='tags',
            field=models.ManyToManyField(to='quickphotos.Tag', blank=True),
        ),
    ]



from django.conf import settings
from django.core.management.base import BaseCommand
from django.db import transaction
from instagram.client import InstagramAPI

from quickphotos.utils import update_photos


@transaction.atomic
def update_user(user, download):
    api = InstagramAPI(
        access_token=settings.INSTAGRAM_ACCESS_TOKEN,
        client_secret=settings.INSTAGRAM_CLIENT_SECRET)

    recent_media, next = api.user_recent_media(user_id=user)
    update_photos(photos=recent_media, download=download)


class Command(BaseCommand):
    help = 'Download and store the latest photos of followed users'

    def add_arguments(self, parser):
        parser.add_argument(
            'users', metavar='user', nargs='+',
            help='Instagram user ID to update')
        parser.add_argument(
            '--download-photos', action='store_true',
            help='Download images from photos and store locally')

    def handle(self, **options):
        for user in options['users']:
            update_user(user=user, download=options['download_photos'])
